import inflection
import os

def modify(name, microservice=None):
    crud_doc = (
        f"## 📦 CRUD de {inflection.pluralize(name).capitalize()}\n"
        f"\n"
        f"- `GET /api/v1/{name}`: Lista todas las {inflection.pluralize(name)}\n"
        f"- `GET /api/v1/{name}/{{id}}`: Obtiene una {name} por ID: UUID\n"
        f"- `POST /api/v1/{name}`: Crea una nueva {name}\n"
        f"- `PUT /api/v1/{name}/{{id}}`: Actualiza una {name} existente por ID: UUID\n"
        f"- `DELETE /api/v1/{name}/{{id}}`: Elimina una {name} por ID: UUID\n"
    )

    if microservice is not None:
        readme_path = f"{microservice}/README.md"
    else:
        readme_path = "README.md"

    # Leer el contenido actual
    with open(readme_path, "r", encoding="utf-8") as file:
        content = file.read()

    # Verificar si ya existe una sección de CRUD
    if f"## 📦 CRUD de {inflection.pluralize(name).capitalize()}" not in content:
        # Agregar al final del archivo
        content += "\n" + crud_doc

        # Guardar el archivo modificado
        with open(readme_path, "w", encoding="utf-8") as file:
            file.write(content)

        print(f"[OK] Se ha agregado la sección CRUD {name} al README.md.")
    else:
        print(f"[INFO] Ya existe una sección CRUD {name} en el README.md.")



from ruamel.yaml.compat import StringIO
from ruamel.yaml import YAML
import os

def modify(name):

    yaml = YAML()
    yaml.preserve_quotes = True

    compose_file = "docker-compose.yaml"

    block = (
        f"  {name}:\n"
        "    build:\n"
        f"      context: ./{name}\n"
        "      dockerfile: Dockerfile\n"
        f"    container_name: {name}-service\n"
        "    volumes:\n"
        f"      - ./{name}:/app\n"
        "    command: uvicorn app.main:run --host 0.0.0.0 --port ${SERVICE_PORT} --reload\n"
        "    networks:\n"
        "      - quipus-network\n"
        "    expose:\n"
        "      - ${SERVICE_PORT}\n"
        "    depends_on:\n"
        "      - globaldb\n"
        "      - dbburbujas\n"
        "      - dbzabala\n"
        "      - dbmafeseguros\n"
    )

    # Leer el contenido actual
    with open(compose_file, "r", encoding="utf-8") as f:
        lines = f.readlines()

    # Buscar el índice de la línea que contiene el servicio nginx
    insert_index = None
    for i, line in enumerate(lines):
        if line.strip().startswith("nginx:") and line.startswith("  "):  # indentación 2 espacios
            insert_index = i
            break

    # Insertar el bloque antes de nginx si no está presente
    if insert_index is not None:
        if f"{name}:" not in "".join(lines):  # Evitar insertar duplicado
            lines.insert(insert_index, block)
            with open(compose_file, "w", encoding="utf-8") as f:
                f.writelines(lines)
            print(f"[OK] Servicio '{name}' insertado correctamente antes de 'nginx'")
        else:
            print(f"[INFO] El servicio '{name}' ya existe en el archivo docker-compose.yaml")
    else:
        print("[ERROR] No se encontró el servicio 'nginx:' en el archivo")

    with open(compose_file, 'r') as file:
        data = yaml.load(file)

    # Verifica si 'nginx' y su sección 'depends_on' existen
    nginx_service = data['services'].get('nginx', {})
    depends_on = nginx_service.get('depends_on', [])

    # Agrega el servicio al nginx
    if name not in depends_on:
        depends_on.append(name)

        # Actualiza el servicio nginx
        data['services']['nginx']['depends_on'] = depends_on

        # Guardar los cambios de vuelta en el archivo
        with open(compose_file, 'w') as file:
            yaml.dump(data, file)

        print(F"[OK] Se ha agregado el servicio '{name}' al 'depends_on' de nginx.")
    else:
        print(f"[INFO] El servicio '{name}' ya está presente en 'depends_on' de nginx.")



from jinja2 import Environment, FileSystemLoader
from importlib.resources import files
import taypiproject
import sys
import os

def create(name):
    # Si el nombre del microservicio ya existe, no lo creamos
    if os.path.exists(name):
        print(f"[INFO] El microservicio '{name}' ya existe.")
        return

    # si no esta ubicado en la carpeta del proyecto que debe contener un docker-compose.yaml para toda la ejecucion
    if not os.path.isfile(f"{os.getcwd()}/docker-compose.yaml"):
        print("[INFO] No se encuentra en la carpeta del proyecto.")
        sys.exit()

    TEMPLATE_DIR = str(files(taypiproject).joinpath("templates"))
    env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))
    context = {
        'name': name
    }


    estructura = [
        "app/components/filter/domain/value_objects",
        "app/components/filter/infrastructure/dto",
        "app/Traits",
        "commands",
        "config",
        "config/translate",
        "config/database",
        "database",
        "middleware",
        "middleware/exceptions",
        "database/migrations",
    ]

    archivos = {
        '.env': 'env.txt',
        'alembic.ini': 'microservice/alembic.ini.txt',
        "command": 'microservice/command.txt',
        'Dockerfile': 'dockerfile.txt',
        'requirements.txt': 'requirements.txt',
        'README.md': 'microservice/readme.txt',
        'app/main.py': 'microservice/main.txt',
        'app/components/filter/domain/filter.py': 'microservice/domain_filter.txt',
        'app/components/filter/domain/value_objects/filter.py': 'microservice/value_object_filter.txt',
        'app/components/filter/infrastructure/dto/input_filter.py': 'microservice/input_filter.txt',
        "app/Traits/filters.py": 'microservice/trait_filter.txt',
        "commands/seeder.py": 'microservice/command_seeder.txt',
        "config/database/settings.py": 'microservice/database_settings.txt',
        "config/database/connections.py": 'microservice/connections.txt',
        "config/secret.py": 'microservice/secret.txt',
        "config/settings.py": 'microservice/settings.txt',
        "config/translate/messages.py": 'microservice/messages.txt',
        "database/env.py": 'microservice/alembic_env.txt',
        "database/script.py.mako": 'microservice/alembic_script.txt',
        "middleware/validation.py": 'microservice/validation.txt',
        "middleware/exceptions/handler.py": 'microservice/handler.txt',
    }

    os.makedirs(name, exist_ok=True)

    # Crear subdirectorios
    for carpeta in estructura:
        os.makedirs(os.path.join(name, carpeta), exist_ok=True)

    # Crear archivos
    for archivo, path_template in archivos.items():
        try:
            content = env.get_template(path_template).render(**context)
        except Exception as e:
            print(f"[ERROR] El template del archivo '{archivo}' no existe: {e}")
            return

        ruta_archivo = os.path.join(name, archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    print(f"[OK] El microservicio '{name}' creado con éxito.")


import argparse

from taypiproject.microservice_generator import create as microservice
from taypiproject.docker_modified import modify as docker_modify
from taypiproject.migration_generator import create as migration
from taypiproject.readme_modified import modify as readme_modify
from taypiproject.main_modified import modify as main_modify
from taypiproject.project_generator import create as project
from taypiproject.git_generator import create as git_create
from taypiproject.entity_generator import create as entity
from taypiproject.site_generator import create as site
from taypiproject.envdocker import create as envdocker_create

def main():
    parser = argparse.ArgumentParser(description="Generador de proyectos FastAPI con docker-compose y base de datos postgres.")
    subparsers = parser.add_subparsers(dest="accion", required=True, help="Acciones disponibles.")
    # Comando para inicializar un nuevo proyecto
    init_parser = subparsers.add_parser("init", help="Iniciar un nuevo proyecto.")
    init_parser.add_argument("name", help="Nombre del proyecto.")

    # Comando para crear un microservicio
    microservice_parser = subparsers.add_parser("microservice", help="Crear un microservicio con Arquitectura Hexagonal.")
    microservice_parser.add_argument("name", help="Nombre del microservicio.")
    microservice_parser.add_argument("entity", help="Nombre de la entidad.")

    # Comando para crear una entity
    entity_parser = subparsers.add_parser("entity", help="Crear una entity con Arquitecura Hexagonal.")
    entity_parser.add_argument("name", help="Nombre de la entity.")

    # Comando para agregar en .env y el Dockerfile
    envdocker_parser = subparsers.add_parser("envdocker", help="Agregar en .env y el Dockerfile.")

    args = parser.parse_args()

    if args.accion == "init":
        print("[INFO] Iniciando un nuevo proyecto...")
        project(args.name)
        git_create("init", args.name)

    elif args.accion == "microservice":
        microservice(args.name)
        docker_modify(args.name)
        site(args.name)
        entity(args.entity, args.name)
        migration(args.entity, args.name)
        main_modify(args.entity, args.name)
        readme_modify(args.entity, args.name)
        git_create("microservice", args.name)

    elif args.accion == "entity":
        entity(args.name)
        migration(args.name)
        main_modify(args.name)
        readme_modify(args.name)

    #comando para agregar en .enc y el Dockerfile
    elif args.accion == "envdocker":
        envdocker_create()

if __name__ == "__main__":
    main()



from jinja2 import Environment, FileSystemLoader
from importlib.resources import files
import taypiproject
import os

def create(name):

    TEMPLATE_DIR = str(files(taypiproject).joinpath("templates"))
    env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))

    estructura = [
        "nginx/ssl",
        "nginx/sites",
        "example_entity/app",
    ]

    archivos = {
        "README.md" : 'project/README.txt',
        ".env" : 'project/env.txt',
        "docker-compose.yaml": 'project/docker-compose.txt',
        "nginx/sites/example_entity.conf": 'project/site_example.txt',
        "example_entity/app/main.py" : 'project/main_example.txt',
        "example_entity/requirements.txt": 'requirements.txt',
        "example_entity/Dockerfile": 'dockerfile.txt',
    }

    # Crear directorio base
    os.makedirs(name, exist_ok=True)

    print(f"[INFO] Creando proyecto '{name}'...")

    # Crear subdirectorios
    for carpeta in estructura:
        os.makedirs(os.path.join(name, carpeta), exist_ok=True)

    # Crear archivos vacíos
    for archivo, path_template in archivos.items():
        try:
            content = env.get_template(path_template).render()
        except Exception as e:
            print(f"[ERROR] El template del archivo'{archivo}' no existe.")
            return

        ruta_archivo = os.path.join(name, archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    print(f"[OK] Proyecto '{name}' creado con exito.")


import os

def create(name):
    archivos = {
        f"{name}.conf" : (
            "server {\n"
        "    #listen 443 ssl;\n"
        "    listen 80;\n"
        f"    server_name {name}.app;\n"
        "    #ssl_certificate /etc/nginx/ssl/certificate.crt; # managed by Certbot\n"
        "    #ssl_certificate_key /etc/nginx/ssl/private.key; #\n"
        "\n"
        "    location / {\n"
        f"        proxy_pass http://{name}:5000;\n"
        "        proxy_set_header Host $host;\n"
        "        proxy_set_header X-Real-IP $remote_addr;\n"
        "        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;\n"
        "        proxy_set_header X-Forwarded-Proto $scheme;\n"
        "    }\n"
        "}\n"
        )
    }

    # Crear archivos vacíos
    for archivo, content in archivos.items():
        ruta_archivo = os.path.join('nginx/sites', archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    print(f"[OK] Se configuro el DNS '{name}.app' agregalo en el archivo host del sistema.")


from pathlib import Path
import subprocess
import time
import sys
import os

def run_command(command, cwd=None):
    subprocess.run(
        command,
        cwd=cwd,
        shell=True,
        check=True,
        stdout=subprocess.DEVNULL,   # No mostrar salida estándar
    )

def create(proccess, name):
    if proccess == "init":
        gitignore = (
            ".env\n"
            ".DS_Store\n"
            ".vscode/\n"
            ".idea/\n"
            "example_entity\n"
            "nginx/sites/example_entity.conf\n"
        )
    elif proccess == "microservice":
        gitignore = (
            "# Entornos virtuales\n"
            "venv/\n"
            ".env\n"
            "\n"
            "# Archivos de Python compilados\n"
            "__pycache__/\n"
            "*.py[cod]\n"
            "*.pyo\n"
            "\n"
            "# Configuración de IDE\n"
            ".vscode/\n"
            ".idea/\n"
            "\n"
            "# Archivos temporales\n"
            "*.log\n"
            ".DS_Store\n"
            "\n"
            "# Build\n"
            "dist/\n"
            "build/\n"
            "*.egg-info/\n"
            "\n"
            "# Archivos de configuración de Docker\n"
            "Dockerfile\n"
            ".dockerignore\n"
        )

    archivos = {
        ".gitignore": gitignore,
        ".gitmodules": "[submodule]\n"
    }

    project_path = Path.cwd() / name
    gitignore_path = project_path / ".gitignore"

    if gitignore_path.exists():
        print("[INFO] El proyecto git ya esta inicializado.")
        sys.exit(0)

    # Crear archivos
    for archivo, content in archivos.items():
        ruta_archivo = os.path.join(name, archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    if proccess == "init":
        tipo = "proyecto"
    elif proccess == "microservice":
        tipo = "microservicio"

    print(f"[OK] Se creo el archivo .gitignore para el {tipo} '{name}' con exito.")

    # Ejecutar comandos de git en el directorio del proyecto
    run_command("git init", cwd=project_path)
    run_command("git add .", cwd=project_path)
    result = subprocess.run(
        "git status --porcelain",
        cwd=project_path,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True
    )

    if result.stdout.strip():
        run_command('git commit -m "Initial commit"', cwd=project_path)
        run_command("git branch -M main", cwd=project_path)
        print("[OK] Commit inicial.")
    else:
        print("[WARN] No hay archivos para commitear.")


    print(f"[OK] Se inicializo el repositorio git para el {tipo} '{name}' con exito.")




from datetime import datetime
import inflection
import uuid
import os
import re

def create(name, microservice = None):

    revision_id = uuid.uuid4().hex[:12]
    timestamp = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f')

    archivos = {
        f"database/migrations/{revision_id}_create_{name}_table.py":(
            f"\"\"\"create {inflection.pluralize(name)} table\n"
            "\n"
            f"Revision ID: {revision_id}\n"
            "Revises:\n"
            f"Create Date: {timestamp}\n"
            "\n"
            "\"\"\"\n"
            "from typing import Sequence, Union\n"
            "\n"
            "from alembic import op\n"
            "import sqlalchemy as sa\n"
            "from sqlalchemy.dialects import postgresql\n"
            "\n"
            "# revision identifiers, used by Alembic.\n"
            f"revision: str = '{revision_id}'\n"
            "down_revision: Union[str, None] = None\n"
            "branch_labels: Union[str, Sequence[str], None] = None\n"
            "depends_on: Union[str, Sequence[str], None] = None\n"
            "\n"
            "\n"
            "def upgrade() -> None:\n"
            f"    op.create_table('{inflection.pluralize(name)}',\n"
            "    sa.Column('id', sa.UUID(), autoincrement=False, nullable=False),\n"
            "    sa.Column('example', sa.VARCHAR(), autoincrement=False, nullable=False),\n"
            "    sa.Column('created_at', postgresql.TIMESTAMP(), server_default=sa.text('now()'), autoincrement=False, nullable=False),\n"
            "    sa.Column('updated_at', postgresql.TIMESTAMP(timezone=True), autoincrement=False, nullable=False),\n"
            "    sa.Column('deleted_at', postgresql.TIMESTAMP(), autoincrement=False, nullable=True),\n"
            f"    sa.PrimaryKeyConstraint('id', name='{inflection.pluralize(name)}_pkey')\n"
            "    )\n"
            f"    op.create_index('ix_{inflection.pluralize(name)}_example', '{inflection.pluralize(name)}', ['example'], unique=False)\n"
            "\n"
            "def downgrade() -> None:\n"
            f"    op.drop_index('ix_{inflection.pluralize(name)}_id', table_name='{inflection.pluralize(name)}')\n"
            f"    op.drop_index('ix_{inflection.pluralize(name)}_example', table_name='{inflection.pluralize(name)}')\n"
            f"    op.drop_table('{inflection.pluralize(name)}')\n"
        )
    }

    # Si existe el microservicio crea el path de la migración dentro del microservicio
    if microservice is not None:
        path = f"{microservice}/database/migrations"
    else:
        path = "database/migrations"

    patron = re.compile(rf"_create_{name}_table")
    for nombre_archivo in os.listdir(path):
        if patron.search(nombre_archivo):
            print(f"[INFO] Ya existe una migración que crea la tabla {inflection.pluralize(name)}")
            return True

    if microservice is not None:
        path = f"{microservice}"
    else:
        path = ""

    # Crear archivos vacíos
    for archivo, content in archivos.items():
        ruta_archivo = os.path.join(path , archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)
            print(f"[OK] Se ha creado la migración {archivo} para crear la tabla {inflection.pluralize(name)}")



from jinja2 import Environment, FileSystemLoader
from importlib.resources import files
import taypiproject
import sys
import os

def create():

    # comprobar que exista la carpeta app
    if not os.path.isdir(f"{os.getcwd()}/app"):
        print("[INFO] No se encuentra en la carpeta del un microservicio.")
        sys.exit()

    TEMPLATE_DIR = str(files(taypiproject).joinpath("templates"))
    env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))

    archivos = {
        '.env': 'env.txt',
        'Dockerfile': 'dockerfile.txt',
    }

    # Crear archivos
    for archivo, path_template in archivos.items():
        try:
            content = env.get_template(path_template).render()
        except Exception as e:
            print(f"[ERROR] El template del archivo '{archivo}' no existe: {e}")
            return

        ruta_archivo = os.path.join('', archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    print(f"[OK] Se crearon los archivo Dockerfile y .env creado con éxito.")


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="taypi-create",
    version="0.3.0",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "taypi-create=taypiproject.cli:main",
        ],
    },
    install_requires=[
        "inflection",
        "ruamel.yaml",
        "Jinja2",
    ],
    description="FastAPI and PostGreSQL.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Juan David Corrales Saldarriaga",
    author_email="sistemas@taypi.com",
    license="Proprietary",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



import os

def modify(name, microservice=None):

    if microservice is not None:
        main_file = f"{microservice}/app/main.py"
    else:
        main_file = "app/main.py"

    import_line = f"from app.{name}.infrastructure.routers.{name}_router import router as {name}_router\n"
    router_line = f"run.include_router({name}_router, prefix=\"/api/v1\", tags=[\"{name.capitalize()}\"])\n"

    # Leer contenido actual
    with open(main_file, "r", encoding="utf-8") as f:
        lines = f.readlines()

    # Verificar si ya existe
    if import_line in lines and router_line in lines:
        print("[INFO] Las líneas ya están en main.py")
    else:
        # === 1. Insertar IMPORT si no existe ===
        if import_line not in lines:
            import_indices = [i for i, line in enumerate(lines) if line.startswith("import") or line.startswith("from")]
            last_import_idx = import_indices[-1] if import_indices else 0
            lines.insert(last_import_idx + 2, import_line + "\n")

        # === 2. Insertar include_router en el lugar correcto ===
        # Buscar todas las líneas que incluyen run.include_router
        router_indices = [i for i, line in enumerate(lines) if "run.include_router" in line]

        if router_indices:
            # Insertar justo después del último include_router
            last_router_idx = router_indices[-1]
            if router_line not in lines:
                lines.insert(last_router_idx + 1, router_line)
        else:
             # Si no hay ningún run.include_router, insertar al final del archivo
            if not lines[-1].endswith("\n"):
                lines[-1] += "\n"
            lines.append("\n" + router_line)

        # Guardar archivo actualizado
        with open(main_file, "w", encoding="utf-8") as f:
            f.writelines(lines)

        print("[OK] main.py actualizado exitosamente.")


from jinja2 import Environment, FileSystemLoader
from importlib.resources import files
import taypiproject
import inflection
import sys
import os

def create(name, microservice = None):

    camel_case_plural = lambda s: inflection.pluralize(s.split('_')[0]) + ''.join(word.capitalize() for word in s.split('_')[1:])
    pascal_case = ''.join(word.capitalize() for word in name.split('_'))
    name_plural = camel_case_plural(name)

    TEMPLATE_DIR = str(files(taypiproject).joinpath("templates"))
    env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))
    context = {
        'name': name,
        'pascal_case': pascal_case,
        'name_plural': name_plural,
        'name_plural_capitalized': name_plural.capitalize(),
    }

    estructura = [
        "application/services",
        "application/use_cases",
        "domain/entities",
        "domain/repositories",
        "domain/value_objects",
        "infrastructure/dto",
        "infrastructure/models",
        "infrastructure/repositories",
        "infrastructure/routers",
        "infrastructure/validations",
    ]

    archivos = {
        f"application/services/{name}_service.py": 'entity/service.txt',
        f"application/use_cases/create_{name}.py": 'entity/use_case_create.txt',
        f"application/use_cases/delete_{name}.py": 'entity/use_case_delete.txt',
        f"application/use_cases/get_{name_plural}.py": 'entity/use_case_get_all.txt',
        f"application/use_cases/get_{name}_by_id.py": 'entity/use_case_get_by_id.txt',
        f"application/use_cases/update_{name}.py": 'entity/use_case_update.txt',
        f"domain/entities/{name}.py": 'entity/entity.txt',
        f"domain/repositories/{name}_repository.py": 'entity/repository.txt',
        f"domain/value_objects/datetime.py": 'entity/value_object_datetime.txt',
        f"infrastructure/dto/input_{name}.py": 'entity/input_dto.txt',
        f"infrastructure/dto/output_{name}.py": 'entity/output_dto.txt',
        f"infrastructure/models/{name}_model.py": 'entity/model.txt',
        f"infrastructure/repositories/{name}_repository_impl.py": 'entity/repository_impl.txt',
        f"infrastructure/routers/{name}_router.py": 'entity/router.txt',
        f"infrastructure/validations/insert.py": 'entity/insert_validation.txt',
        f"infrastructure/validations/update.py": 'entity/update_validation.txt',
    }

    if microservice is None and not os.path.isfile(f"{os.getcwd()}/Dockerfile"):
        print("[INFO] No se encuentra en la carpeta de un microservicio.")
        sys.exit()

    if microservice is not None:
        path = f"{microservice}/app/{name}"
    else:
        path = f"app/{name}"

    # Si la entidad ya existe, no la crea
    if os.path.exists(path):
        print(f"[INFO] La entidad {name} ya existe.")
        return

    if microservice is not None:
        os.makedirs(microservice + '/app/' + name, exist_ok=True)
    else:
        os.makedirs(path, exist_ok=True)

    # Crear subdirectorios
    for carpeta in estructura:
        os.makedirs(os.path.join(path, carpeta), exist_ok=True)

    # Crear archivos vacíos
    for archivo, path_template in archivos.items():
        try:
            content = env.get_template(path_template).render(**context)
        except Exception as e:
            print(f"[ERROR] No se pudo crear el archivo {archivo}: {e}")
            return

        ruta_archivo = os.path.join(path, archivo)
        with open(ruta_archivo, "w", encoding="utf-8") as f:
            f.write(content)

    print(f"[OK] Entidad {name} creada con éxito.")
